/***************************************************************************
 *   Copyright (C) 2004 by Reto Schoelly,,,                                *
 *   reto@reto-schoelly.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "defines.h"
 
#include "MainDialog.h"
#include "OpenGLWidget.h"
#include "setupgamedlg.h"

#include "mahjonggboard.h"
#include "configuregamedialog.h"
#include "manualdialog.h"
#include "hiscorelist.h"
#include "hiscorewidget.h"

#include "qlabel.h"
#include "qcombobox.h"
#include "qlineedit.h"

#include "mahjongggl.h"

#include "time.h"

#include "qmessagebox.h"
#include "qfiledialog.h"
#include "qfileinfo.h"
#include "qstring.h"

#include "gamedata_path.h"

extern int game_time;
extern char HISCORE_FILENAME[300];

MainDialog :: MainDialog(QWidget *parent, const char *name)
{
	connect((QObject *)customMenuBar, SIGNAL(sendNewGame()), this, SLOT(NewGame()));
	connect((QObject *)customMenuBar, SIGNAL(sendConfigureGame()), this, SLOT(ConfigureGame()));
	connect((QObject *)customMenuBar, SIGNAL(sendReorderGame()), this, SLOT(ReorderGame()));
	connect((QObject *)customMenuBar, SIGNAL(sendUndo()), this, SLOT(Undo()));
	connect((QObject *)customMenuBar, SIGNAL(sendManual()), this, SLOT(Manual()));
	connect((QObject *)customMenuBar, SIGNAL(sendQuit()), this, SLOT(close()));
	connect((QObject *)customMenuBar, SIGNAL(sendToggleShadows()), this, SLOT(ToggleShadows()));
	connect((QObject *)customMenuBar, SIGNAL(sendSolveTurn()), this, SLOT(SolveTurn()));
	connect((QObject *)customMenuBar, SIGNAL(sendHint()), this, SLOT(Hint()));
	connect((QObject *)customMenuBar, SIGNAL(sendHiscores()), this, SLOT(Hiscores()));
	connect((QObject *)GLWidget, SIGNAL(sendUpdatePairs()), this, SLOT(UpdatePairs()));
	connect((QObject *)GLWidget, SIGNAL(sendRandomSeed(QString &)), this, SLOT(UpdateRandomSeed(QString &)));
	
	ManualDialog dlg(this, 0);
	dlg.exec();
	GLWidget->updateGL();
}

MainDialog :: ~MainDialog()
{
}

void MainDialog :: Hint()
{
	GLWidget->mgl->board->hint();
	GLWidget->updateGL();
}

void MainDialog :: SolveTurn()
{
	GLWidget->mgl->board->solveTurn();
	GLWidget->updateGL();
}

void MainDialog :: UpdateRandomSeed(QString &str)
{
	textLabel_GameNr->setText(str);
}

void MainDialog :: Hiscores()
{
	if(!HISCORE_FILENAME) return;
	
	QString str, number;
	HiscoreList list;
	if(list.readFile(HISCORE_FILENAME) == -1)
	{
		QMessageBox::information(this, "Could not load hiscore table", "No hiscores have been made under this account! Be the first :-))",QMessageBox::Ok, 0, 0);
		return;
	}
	list.sort();
	
	HiscoreWidget hiscore(this, 0);
	
	
	str = "";
	str.append(list.name[0]);
	number.setNum(list.score[0]);
	hiscore.lineEdit_name1->setText(str);
	hiscore.lineEdit_score1->setText(number);
	
	str = "";
	str.append(list.name[1]);
	number.setNum(list.score[1]);
	hiscore.lineEdit_name2->setText(str);
	hiscore.lineEdit_score2->setText(number);
	
	str = "";
	str.append(list.name[2]);
	number.setNum(list.score[2]);
	hiscore.lineEdit_name3->setText(str);
	hiscore.lineEdit_score3->setText(number);
	
	str = "";
	str.append(list.name[3]);
	number.setNum(list.score[3]);
	hiscore.lineEdit_name4->setText(str);
	hiscore.lineEdit_score4->setText(number);
	
	str = "";
	str.append(list.name[4]);
	number.setNum(list.score[4]);
	hiscore.lineEdit_name5->setText(str);
	hiscore.lineEdit_score5->setText(number);
	
	str = "";
	str.append(list.name[5]);
	number.setNum(list.score[5]);
	hiscore.lineEdit_name6->setText(str);
	hiscore.lineEdit_score6->setText(number);
	
	str = "";
	str.append(list.name[6]);
	number.setNum(list.score[6]);
	hiscore.lineEdit_name7->setText(str);
	hiscore.lineEdit_score7->setText(number);
		
	str = "";
	str.append(list.name[7]);
	number.setNum(list.score[7]);
	hiscore.lineEdit_name8->setText(str);
	hiscore.lineEdit_score8->setText(number);
	
	str = "";
	str.append(list.name[8]);
	number.setNum(list.score[8]);
	hiscore.lineEdit_name9->setText(str);
	hiscore.lineEdit_score9->setText(number);
	
	str = "";
	str.append(list.name[9]);
	number.setNum(list.score[9]);
	hiscore.lineEdit_name10->setText(str);
	hiscore.lineEdit_score10->setText(number);
	
	
	hiscore.exec();
}

void MainDialog :: NewGame()
{
	QString str;
	QString layout;
	QString tileset;
	
	SetupGameDlg dlg(this, 0);
	
	dlg.exec();
	
	if(dlg.comboBoxLayouts->currentItem() == 0)
	{
		layout = "turtle.layout";
		str = "Playing TURTLE layout, game number: ";
		textLabel_PlayingNr->setText(str);
	}
	else if(dlg.comboBoxLayouts->currentItem() == 1)
	{
		layout = "castle.layout";
		str = "Playing FAST CASTLE layout, game number: ";
		textLabel_PlayingNr->setText(str);
	}
	
	if(dlg.comboBoxTilesets->currentItem() == 0)
	{
		tileset = "default.tileset";
	}
	else if(dlg.comboBoxTilesets->currentItem() == 1)
	{
		tileset = "traditional.tileset";
	}
	else if(dlg.comboBoxTilesets->currentItem() == 2)
	{
		tileset = "fancyletters.tileset";
	}

	if(dlg.comboBoxBackgrounds->currentItem() == 0)
	{
		GLWidget->mgl->createBackground("gra/black.background.bmp");
	}
	else if(dlg.comboBoxBackgrounds->currentItem() == 1)
	{
		GLWidget->mgl->createBackground("gra/warm.background.bmp");
	}
	else if(dlg.comboBoxBackgrounds->currentItem() == 2)
	{
		GLWidget->mgl->createBackground("gra/blue.background.bmp");
	}
	
	GLWidget->mgl->makeBoard(0, layout.latin1(), tileset.latin1());
	textLabel_GameNr->setText(str.setNum(GLWidget->mgl->board->getRandomSeed()));
	GLWidget->updateGL();
	
	game_time = time(NULL);
}

void MainDialog :: ConfigureGame()
{
	QString str;
	
	ConfigureGameDialog *dlg = new ConfigureGameDialog(this, 0);
	dlg->exec();
	
	dlg->lineEdit_GameNum->text().toInt();
	GLWidget->mgl->makeBoard(dlg->lineEdit_GameNum->text().toInt(), "turtle.layout", "default.hires");

	textLabel_GameNr->setText(str.setNum(GLWidget->mgl->board->getRandomSeed()));
	GLWidget->updateGL();
	
	delete dlg;
}


void MainDialog :: ReorderGame()
{
	GLWidget->mgl->board->reorder();
	GLWidget->updateGL();
}


void MainDialog :: Undo()
{
	GLWidget->mgl->board->undo();
	GLWidget->updateGL();
}

void MainDialog :: Manual()
{
	ManualDialog dlg(this, 0);
	dlg.exec();
}


void MainDialog :: UpdatePairs()
{
}


void MainDialog :: ToggleShadows()
{
	if(GLWidget->mgl->shadow) GLWidget->mgl->shadow = false;
	else GLWidget->mgl->shadow = true;
}
